import * as React from 'react';
import {getMimeTypeDescription, VideoFile} from '@/utils/blob';
import {formatFileSize, formatNumeral, formatSecondsDurationConcise} from '@/utils/numeral';
import {useVideoControls} from '@tesseract-design/web-blob-react';

export interface VideoFilePreviewProps {
  file: VideoFile;
}

export const VideoFilePreview = React.forwardRef<HTMLVideoElement, VideoFilePreviewProps>(({
  file: f,
}, forwardedRef) => {
  const defaultRef = React.useRef<HTMLVideoElement>(null);
  const mediaControllerRef = forwardedRef ?? defaultRef;
  const {
    seekRef,
    volumeRef,
    isPlaying,
    adjustVolume,
    playMedia,
    resetVideo,
    startSeek,
    endSeek,
    setSeek,
    updateSeekFromPlayback,
    refreshControls,
    durationDisplay = 0,
    currentTimeDisplay = 0,
    seekTimeDisplay = 0,
    isSeeking,
    isSeekTimeCountingDown,
    toggleSeekTimeCountMode,
  } = useVideoControls({
    mediaControllerRef,
  });

  const finalSeekTimeDisplay = isSeekTimeCountingDown ? (durationDisplay - seekTimeDisplay) : seekTimeDisplay;
  const finalCurrentTimeDisplay = isSeekTimeCountingDown ? (durationDisplay - currentTimeDisplay) : currentTimeDisplay;

  return (
    <div className="flex gap-4 w-full h-full">
      <div className={`h-full w-1/3 flex-shrink-0`}>
        {
          typeof f.metadata?.previewUrl === 'string'
          && (
            <div
              className="w-full h-full bg-black flex flex-col items-stretch"
              data-testid="preview"
            >
              <div className="w-full flex-auto relative">
              <video
                className="absolute w-full h-full top-0 left-0 block object-center object-contain flex-auto"
                ref={mediaControllerRef as React.RefObject<HTMLVideoElement>}
                onLoadedMetadata={refreshControls}
                onDurationChange={refreshControls}
                onEnded={resetVideo}
                onTimeUpdate={updateSeekFromPlayback}
              >
                <source
                  src={f.metadata.previewUrl}
                  type={f.type}
                />
              </video>
              </div>
              <div className="w-full flex-shrink-0 h-10 flex gap-4">
                <button
                  onClick={playMedia}
                  className="w-10 h-full flex-shrink-0"
                  type="button"
                >
                  {isPlaying ? '⏸' : '▶'}
                </button>
                <div className="flex-auto w-full flex items-center gap-1 font-mono text-sm relative">
                  <button
                    className="absolute overflow-hidden w-12 opacity-0 h-10"
                    onClick={toggleSeekTimeCountMode}
                    title="Toggle Seek Time Count Mode"
                    type="button"
                  >
                    Toggle Seek Time Count Mode
                  </button>
                  <span
                    className="font-mono before:block before:content-[attr(title)] contents"
                    title={
                      `${
                        isSeekTimeCountingDown ? '-' : '+'
                      }${
                        isSeeking
                          ? formatSecondsDurationConcise(finalSeekTimeDisplay)
                          : formatSecondsDurationConcise(finalCurrentTimeDisplay)
                      }`
                    }
                  >
                    <input
                      type="range"
                      className="flex-auto w-full"
                      ref={seekRef}
                      onMouseDown={startSeek}
                      onMouseUp={endSeek}
                      onChange={setSeek}
                      defaultValue="0"

                    />
                  </span>
                  <span>
                    {formatSecondsDurationConcise(durationDisplay)}
                  </span>
                </div>
                <input
                  type="range"
                  ref={volumeRef}
                  max={1}
                  min={0}
                  onChange={adjustVolume}
                  step="any"
                  className="flex-shrink-0 w-12"
                  defaultValue="1"
                  title="Volume"
                />
              </div>
            </div>
          )
        }
      </div>
      <dl className="w-2/3 flex-shrink-0 m-0" data-testid="infoBox">
        <div className="w-full">
          <dt className="sr-only">
            Name
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={f.name}
          >
            {f.name}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Type
          </dt>
          <dd
            title={f.type}
            className="m-0 w-full text-ellipsis overflow-hidden"
          >
            {getMimeTypeDescription(f.type, f.name)}
          </dd>
        </div>
        <div className="w-full">
          <dt className="sr-only">
            Size
          </dt>
          <dd
            className="m-0 w-full text-ellipsis overflow-hidden"
            title={`${formatNumeral(f.size ?? 0)} bytes`}
          >
            {formatFileSize(f.size)}
          </dd>
        </div>
        {
          typeof f.metadata?.width === 'number'
          && typeof f.metadata?.height === 'number'
          && (
            <div>
              <dt className="sr-only">
                Pixel Dimensions
              </dt>
              <dd
                className="m-0 w-full text-ellipsis overflow-hidden"
              >
                {formatNumeral(f.metadata.width)}&times;{formatNumeral(f.metadata.height)} pixels
              </dd>
            </div>
          )
        }
      </dl>
    </div>
  );
});

VideoFilePreview.displayName = 'VideoFilePreview';
