import * as React from 'react';

export interface UseVideoControlsOptions {
  mediaControllerRef: React.Ref<HTMLVideoElement>;
}

export const useVideoControls = ({
  mediaControllerRef,
}: UseVideoControlsOptions) => {
  const seekRef = React.useRef<HTMLInputElement>(null);
  const volumeRef = React.useRef<HTMLInputElement>(null);
  const [isPlaying, setIsPlaying] = React.useState(false);
  const [isSeeking, setIsSeeking] = React.useState(false);
  const [currentTimeDisplay, setCurrentTimeDisplay] = React.useState<number>();
  const [seekTimeDisplay, setSeekTimeDisplay] = React.useState<number>();
  const [durationDisplay, setDurationDisplay] = React.useState<number>();
  const [isSeekTimeCountingDown, setIsSeekTimeCountingDown] = React.useState(false);

  const refreshControls: React.ReactEventHandler<HTMLVideoElement> = (e) => {
    const { currentTarget: mediaController } = e;
    const { current: seek } = seekRef;
    if (!seek) {
      return;
    }

    setCurrentTimeDisplay(mediaController.currentTime);
    setDurationDisplay(mediaController.duration);
  };

  const playMedia = React.useCallback(() => {
    setIsPlaying((p) => !p);
  }, []);

  const startSeek: React.MouseEventHandler<HTMLInputElement> = React.useCallback(() => {
    setIsSeeking(true);
  }, []);

  const doSetSeek = (thisElement: HTMLInputElement, mediaController: HTMLVideoElement) => {
    mediaController.currentTime = thisElement.valueAsNumber;
  };

  const setSeek: React.ChangeEventHandler<HTMLInputElement> = React.useCallback((e) => {
    if (!(typeof mediaControllerRef === 'object' && mediaControllerRef !== null)) {
      return;
    }

    const { current: mediaController } = mediaControllerRef;
    if (!mediaController) {
      return;
    }

    const { currentTarget: thisElement } = e;
    setSeekTimeDisplay(thisElement.valueAsNumber);

    if (isSeeking) {
      return;
    }

    doSetSeek(thisElement, mediaController);
  }, [mediaControllerRef, isSeeking]);

  const endSeek: React.MouseEventHandler<HTMLInputElement> = React.useCallback((e) => {
    if (!(typeof mediaControllerRef === 'object' && mediaControllerRef !== null)) {
      return;
    }

    const { current: mediaController } = mediaControllerRef;

    if (!mediaController) {
      return;
    }

    const { currentTarget: thisElement } = e;
    setIsSeeking(false);
    doSetSeek(thisElement, mediaController);
  }, [mediaControllerRef]);

  const resetVideo: React.ReactEventHandler<HTMLVideoElement> = React.useCallback((e) => {
    const videoElement = e.currentTarget;
    setIsPlaying(false);
    videoElement.currentTime = 0;
  }, []);

  const updateSeekFromPlayback: React.ReactEventHandler<HTMLVideoElement> = React.useCallback((e) => {
    if (!seekRef.current) {
      return;
    }

    if (isSeeking) {
      return;
    }

    const { current: seek } = seekRef;

    if (!seek) {
      return;
    }

    const videoElement = e.currentTarget;
    const currentTime = videoElement.currentTime;
    setCurrentTimeDisplay(currentTime);
    seek.value = String(currentTime);
  }, [isSeeking]);

  const adjustVolume: React.ChangeEventHandler<HTMLInputElement> = React.useCallback((e) => {
    if (!(typeof mediaControllerRef === 'object' && mediaControllerRef !== null)) {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }
    const { value } = e.currentTarget;
    mediaControllerRef.current.volume = Number(value);
  }, [mediaControllerRef]);

  const toggleSeekTimeCountMode: React.MouseEventHandler<HTMLButtonElement> = React.useCallback((e) => {
    e.preventDefault();
    setIsSeekTimeCountingDown((b) => !b);
  }, []);

  React.useEffect(() => {
    if (!(typeof mediaControllerRef === 'object' && mediaControllerRef !== null)) {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }

    if (isPlaying) {
      void mediaControllerRef.current.play();
      return
    }

    mediaControllerRef.current.pause();
  }, [isPlaying, mediaControllerRef]);

  React.useEffect(() => {
    if (!seekRef.current) {
      return;
    }

    const { current: seek } = seekRef;
    if (!seek) {
      return;
    }

    seek.value = String(currentTimeDisplay);
  }, [currentTimeDisplay]);

  React.useEffect(() => {
    if (!seekRef.current) {
      return;
    }

    const { current: seek } = seekRef;
    if (!seek) {
      return;
    }

    seek.max = String(durationDisplay);
  }, [durationDisplay]);

  React.useEffect(() => {
    if (!volumeRef.current) {
      return;
    }

    if (!(typeof mediaControllerRef === 'object' && mediaControllerRef !== null)) {
      return;
    }

    if (!mediaControllerRef.current) {
      return;
    }

    const { current: mediaController } = mediaControllerRef;
    const { current: volume } = volumeRef;
    volume.value = String(mediaController.volume);
  }, [mediaControllerRef]);

  return React.useMemo(() => ({
    seekRef,
    volumeRef,
    isPlaying,
    refreshControls,
    adjustVolume,
    resetVideo,
    startSeek,
    endSeek,
    setSeek,
    updateSeekFromPlayback,
    playMedia,
    durationDisplay,
    currentTimeDisplay,
    seekTimeDisplay,
    isSeeking,
    isSeekTimeCountingDown,
    toggleSeekTimeCountMode,
  }), [
    isPlaying,
    isSeeking,
    adjustVolume,
    playMedia,
    resetVideo,
    startSeek,
    endSeek,
    setSeek,
    updateSeekFromPlayback,
    durationDisplay,
    currentTimeDisplay,
    seekTimeDisplay,
    isSeekTimeCountingDown,
    toggleSeekTimeCountMode,
  ]);
};
